/**********************************************************************
 *   Motorola MCF5307 ColdFire System Integration Module Header File
 *
 *   Developed by       : David Fiddes, D.J.Fiddes@hw.ac.uk
 *                        http://www.calm.hw.ac.uk/davidf/coldfire/
 *
 *   Cobbled together from MCF5307UM/AD and MCF5206.h
 *
 **********************************************************************/
 
#ifndef __MCF5307_h
#define __MCF5307_h

/*
 * Cache and Access Control Register routines and defintions
 */

/* #defines will appear when complete */ 

/* Absolute Chip Select structure (similar to other ColdFire family) */
typedef struct
{
  unsigned16 csar;        /* Chip-Select X Base Address Register, 16-bit, R/W (00-01) */
  unsigned8  pack01[2];   /* 02-03 */
  unsigned32 csmr;        /* Chip-Select X Address Mask Register, 32-bit, R/W (04-07) */
  unsigned8  pack02[2];   /* 08-09 */
  unsigned16 cscr;        /* Chip-Select X Control Register, 16-bit, R/W (0A-0B) */
} csstruct_abs;

/* Relative Chip Select structure - relative to csbar */
typedef struct
{
  unsigned32 csmr;        /* Chip-Select X Address Mask Register, 32-bit, R/W (00-03) */
  unsigned8  pack01[2];   /* 04-05 */
  unsigned16 cscr;        /* Chip-Select X Control Register, 16-bit, R/W (06-07) */
  unsigned8  pack02[4];   /* 08-0B */
} csstruct_rel;


/* #defines will appear when complete */ 

typedef struct
{
  unsigned16 tmr;         /* TIMERx Mode Register, 16-bit, R/W (00-01) */
  unsigned8  pack01[2];   /* 02-03 */
  unsigned16 trr;         /* TIMERx Mode Register, 16-bit, R/W (04-05) */
  unsigned8  pack02[2];   /* 06-07 */
  unsigned16 tcr;         /* TIMERx Capture Register, 16-bit, R (08-09) */
  unsigned8  pack03[2];   /* 0A-0B */
  unsigned16 tcn;         /* TIMERx Counter, 16-bit, R/W (0C-0D) */
  unsigned8  pack04[3];   /* 0E-10 */
  unsigned8  ter;         /* TIMERx Event Register, 8-bit, R/W (11) */
} tmrstruct;
  

/* #defines will appear when complete */ 

typedef struct
{
  unsigned8 umr;          /* UARTx Mode Register, 8-bit, R/W (00) */
  unsigned8 pack01[3];    /* 01-03 */
  unsigned8 usr;          /* UARTx Status Register, 8-bit, R (04) */
  unsigned8 pack02[7];    /* 05-0B */
  unsigned8 urbuf;        /* UARTx Receiver Buffer, 8-bit, R (0C) */
  unsigned8 pack03[3];    /* 0D-0F */
  unsigned8 uipcr;        /* UARTx Input Port Change Register, 8-bit, R (10) */
  unsigned8 pack04[3];    /* 11-13 */
  unsigned8 uisr;         /* UARTx Interrupt Status Register, 8-bit, R (14) */
  unsigned8 pack05[3];    /* 15-17 */
  unsigned8 ubg1;         /* UARTx Baud Rate Generator PreScale MSB, 8-bit, R/W (18) */
  unsigned8 pack06[3];    /* 19-1B */
  unsigned8 ubg2;         /* UARTx Baud Rate Generator PreScale LSB, 8-bit, R/W (1C) */
  unsigned8 pack07[19];   /* 1D-2F */
  unsigned8 uivr;         /* UARTx Interrupt Vector Register, 8-bit, R/W (30) */
  unsigned8 pack08[3];    /* 31-33 */
  unsigned8 uip;          /* UARTx Input Port Register, 8-bit, R (34) */
} UARTReadStruct;

typedef struct
{
  unsigned8 umr;          /* UARTx Mode Register, 8-bit, R/W (00) */
  unsigned8 pack01[3];    /* 01-03 */
  unsigned8 ucsr;         /* UART1 Clock Select Register, 8-bit, W (04) */
  unsigned8 pack02[3];    /* 05-07 */
  unsigned8 ucr;          /* UARTx Command Register, 8-bit, W (08) */
  unsigned8 pack2a[3];    /* 09-0B */
  unsigned8 utbuf;        /* UARTx Transmitter Buffer, 8-bit, W (0C) */
  unsigned8 pack03[3];    /* 0D-0F */
  unsigned8 uacr;         /* UARTx Auxiliary Control Register, 8-bit, W (10) */
  unsigned8 pack04[3];    /* 11-13 */
  unsigned8 uimr;         /* UARTx Interrupt Mask Register, 8-bit, W (14) */
  unsigned8 pack05[3];    /* 15-17 */
  unsigned8 ubg1;         /* UARTx Baud Rate Generator PreScale MSB, 8-bit, R/W (18) */
  unsigned8 pack06[3];    /* 19-1B */
  unsigned8 ubg2;         /* UARTx Baud Rate Generator PreScale LSB, 8-bit, R/W (1C) */
  unsigned8 pack07[19];   /* 1D-2F */
  unsigned8 uivr;         /* UARTx Interrupt Vector Register, 8-bit, R/W (30) */
  unsigned8 pack08[7];    /* 31-37 */
  unsigned8 uop1;         /* UARTx Output Port Bit Set Command, 8-bit, W (38) */
  unsigned8 pack09[3];    /* 39-3B */
  unsigned8 uop0;         /* UARTx Output Port Bit Reset Command, 8-bit, W (3C) */
} UARTWriteStruct;

typedef union
{
  UARTReadStruct  read;
  UARTWriteStruct write; 
  unsigned8 pack01[0x40];  
} uartstruct;


/* #defines will appear when complete */ 

typedef struct
{
  unsigned8 madr;         /* M-BUS Address Register, 8-bit, R/W (00) */
  unsigned8 pack01[3];    /* 01-03 */
  unsigned8 mfdr;         /* M-BUS Frequency Divider Register, 8-bit, R/W (04) */
  unsigned8 pack02[3];    /* 05-07 */
  unsigned8 mbcr;         /* M-BUS Control Register, 8-bit, R/W (08) */
  unsigned8 pack03[3];    /* 09-0B */
  unsigned8 mbsr;         /* M-BUS Status Register, 8-bit, R/W (0C) */
  unsigned8 pack04[3];    /* 0D-0F */
  unsigned8 mbdr;         /* M-BUS Data I/O Register, 8-bit, R/W (10) */
} mbusstruct;


/* DMA Channel */

typedef struct
{
  unsigned32 sar;         /* DMA Source Address Register, 32-bit, R/W (00-03) */
  unsigned32 dar;         /* DMA Destination Address Register, 32-bit, R/W (04-07) */
  unsigned16 dcr;         /* DMA Control Register, 16-bit, R/W (08-09) */
  unsigned8  pack01[2];   /* 0A-0B */
  unsigned16 bcr;         /* DMA Byte Count Regsiter, 16-bit, R/W (0C-0D) */
  unsigned8  pack02[2];   /* 0E-0F */
  unsigned8  dsr;         /* DMA Status Register, 8-bit, R/W (10) */
  unsigned8  pack03[3];   /* 11-13 */
  unsigned8  divr;        /* DMA Interrupt Vector Register, 8-bit, R/W (14) */
  unsigned8  pack04[0x2B];/* 15-3F */
} dmastruct;


/* System Integration Module */


/* #defines will appear when complete */ 

typedef struct
{
  unsigned8  rsr;         /* Reset Status Register, 8-bit, R/W (000) */
  unsigned8  sypcr;       /* System Protection Control Register, 8-bit, R/W (001) */
  unsigned8  swivr;       /* Software Watchdog Interrupt Vector Register, 8-bit, W (002) */
  unsigned8  swsr;        /* Software Watchdog Service Register, 8-bit, W (003) */
  unsigned16 par;         /* Pin Assignment Register, 16-bit, R/W (004) */
  unsigned8  irqpar;      /* Interrupt Assignment Register, 8-bit, R/W (006) */
  unsigned8  pack01;      /* 007-007 */
  unsigned8  pllcr;       /* PLL Control Register, 8-bit, R/W (008) */
  unsigned8  pack02[3];   /* 009-00B */
  unsigned8  mpark;       /* Bus Master Control Register, 8-bit, R/W (00C) */
  unsigned8  pack03[0x33];/* 00D-03F */
  unsigned32 ipr;         /* Interrupt Pending Register, 32-bit, R (040-043) */
  unsigned32 imr;         /* Interrupt Mask Register, 32-bit, R/W (044-047) */
  unsigned8  pack04[3];   /* 048-04A */
  unsigned8  avcr;        /* Auto Vector Control Register, 8-bit, R/W (04B) */
  unsigned8  icr0;        /* Interrupt Control Register Ext0, 8-bit, R/W (04C) */
  unsigned8  icr1;        /* Interrupt Control Register Ext1, 8-bit, R/W (04D) */
  unsigned8  icr2;        /* Interrupt Control Register Ext2, 8-bit, R/W (04E) */
  unsigned8  icr3;        /* Interrupt Control Register Ext3, 8-bit, R/W (04F) */
  unsigned8  icr4;        /* Interrupt Control Register Ext4, 8-bit, R/W (050) */
  unsigned8  icr5;        /* Interrupt Control Register Ext5, 8-bit, R/W (051) */
  unsigned8  icr6;        /* Interrupt Control Register Ext6, 8-bit, R/W (052) */
  unsigned8  icr7;        /* Interrupt Control Register Ext7, 8-bit, R/W (053) */
  unsigned8  icr8;        /* Interrupt Control Register Ext8, 8-bit, R/W (054) */
  unsigned8  icr9;        /* Interrupt Control Register Ext9, 8-bit, R/W (055) */
  unsigned8  icr10;       /* Interrupt Control Register Ext10, 8-bit, R/W (056) */
  unsigned8  icr11;       /* Interrupt Control Register Ext11, 8-bit, R/W (057) */
  unsigned8  pack05[0x28];/* 058-07F */

/** Chip Select Registers **/
  
  csstruct_abs     cs_abs[2];    /* Absolute Chip-Select registers (080-097) */
  unsigned8        csbar;        /* Relative Chip-Select Base Register (098) */
  unsigned8        pack06[3];    /* 099-9B */
  csstruct_rel     cs_rel[6];    /* Relative Chip-Select registers (09C-0E3) */
  unsigned8        pack07[0x1C]; /* 0E4-0FF */


/** DRAM Controller Registers **/
  
  unsigned16 dcr;         /* DRAM Controller Register, 16-bit, R/W (100-101) */
  unsigned8  pack08[6];   /* 102-107 */
  
  unsigned32 dacr0;       /* DRAM Controller Bank 0 Address & Control Register, 32-bit, R/W (108-10B) */
  unsigned32 dmr0;        /* DRAM Controller Bank 0 Mask Register, 32-bit, R/W (10C-10F) */
  unsigned32 dacr1;       /* DRAM Controller Bank 1 Address & Control Register, 32-bit, R/W (110-113) */
  unsigned32 dmr1;        /* DRAM Controller Bank 1 Mask Register, 32-bit, R/W (114-117) */


/** General Purpose I/O **/

  unsigned8 pack09[0x28]; /* 118-13F */
  tmrstruct       timer1; /* Timer 1 Module (140-151) */
  unsigned8 pack10[0x2E]; /* 152-17F */
  tmrstruct       timer2; /* Timer 2 Module (180-191) */
  unsigned8 pack11[0x2E]; /* 192-1BF */
  uartstruct      uart1;  /* UART 1 Module (1C0-1FF) */
  uartstruct      uart2;  /* UART 2 Module (200-23F) */
  unsigned8 pack12[4];    /* 240-243 */
  unsigned16      paddr;  /* Port A Data Direction Register, 16-bit, R/W (244-245) */
  unsigned8 pack13[2];    /* 246-247 */
  unsigned16      padat;  /* Port A Data Direction Register, 16-bit, R/W (248-249) */
  unsigned8 pack14[0x36]; /* 24A-27F */
  mbusstruct      mbus;   /* MBUS Module (280 - 290) */
  unsigned8 pack15[0x6F]; /* 291 - 2FF */
  dmastruct       dma[4]; /* 4 DMA Channels (300 - 3FF) */

} SIM5307;

extern volatile SIM5307 *sim;  /* System integration module */

#endif /* __MCF5307_h */
